package edu.harvard.mgh.purcell.gplink.base_form;

import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;

import edu.harvard.mgh.purcell.gplink.forms.*;

/**
 * An extention of JDialog to modify thresholds in the plink command.
 * @author Kathe Todd-Brown
 */
@SuppressWarnings("serial")
public class ThresholdDialog extends JDialog {
	
	/************* Default parameters ***/
	/**
	 * The default minor allelic frequency
	 */
	private static Double MAF_DEFAULT;
	
	/**
	 * The default maximum minor allelic frequency
	 */
	private static Double MAXMAF_DEFAULT;
	
	/**
	 * The default maximum per-SNP missing
	 */
	private static Double GENO_DEFAULT;
	
	/**
	 * The default maximum per-person missing
	 */
	private static Double MIND_DEFAULT;
	
	/**
	 * The form that this dialog is attached to
	 */
	private Forms parent;
	
	/**
	 * Fields for user entered thresholds
	 */
	private GeneralUtil.UpdatingTextField maf, maxMaf, geno,
		mind, hwe, mendell, mendell2;
	
	/**
	 * Buttons to flag user specified thresholds
	 */
	private GeneralUtil.OptionalSelectButton mafButton, maxMafButton, genoButton, 
		hweButton, mendellButton, mindButton;
	
	/**
	 * Buttons to operate this dialog
	 */
	private JButton okButton, resetButton, cancelButton;
	
	/**
	 * Process the dialog to command line.
	 */
	ActionListener process = new ActionListener(){
		public void actionPerformed(ActionEvent e) {
			//clear any old threshold values
			parent.resetThreshold();
			//if the flag is selescted then put it in the threshold
			if(mafButton.isSelected())
				parent.addThreshold(" --maf " + maf.getText());
			if(maxMafButton.isSelected())
				parent.addThreshold(" --max-maf " + maxMaf.getText());
			if(genoButton.isSelected())
				parent.addThreshold(" --geno " + geno.getText());
			if(mindButton.isSelected())
				parent.addThreshold(" --mind " + mind.getText());
			if(hweButton.isSelected())
				parent.addThreshold(" --hwe " + hwe.getText());
			if(mendellButton.isSelected())
				parent.addThreshold(" --me " + mendell.getText() + " " + mendell2.getText());
			//put the ok buttons back on the parent form
			parent.isValidCommand();
			//We are done!
			dispose();
		}
	};
	
	/**
	 * Reset all the defaults
	 */
	ActionListener reset = new ActionListener(){
		public void actionPerformed(ActionEvent e) {
			maf.setText(MAF_DEFAULT.toString());
			maxMaf.setText(MAXMAF_DEFAULT.toString());
			geno.setText(GENO_DEFAULT.toString());
			mind.setText(MIND_DEFAULT.toString());
			hwe.setText("");
			mendell.setText("");
			mendell2.setText("");
		}
		
	};
	
	/**
	 * Cancel the dailog and throw it away without changing things
	 */
	ActionListener cancel = new ActionListener(){
		public void actionPerformed(ActionEvent e) {
			dispose();
		}
	};
	
	/**
	 * Initalize all the variables and fields.
	 *
	 */
	private void initalize(){
		//set the defaults depending on what test we are running
		if(parent.getClass().equals(RecodeForm.class)
				|| parent.getClass().equals(MergeForm.class)){
			MAF_DEFAULT = 0.0;
			GENO_DEFAULT = 1.0;
			MIND_DEFAULT = 1.0;
		} else {
			MAF_DEFAULT = 0.01;
			GENO_DEFAULT = 0.1;
			MIND_DEFAULT = 0.1;
		}
		MAXMAF_DEFAULT = 1.0;
		
		//initalize the fields
		maf = new GeneralUtil.UpdatingTextField(
				MAF_DEFAULT.toString(), 10, null);
		maxMaf = new GeneralUtil.UpdatingTextField(
				MAXMAF_DEFAULT.toString(), 10, null);
		geno = new GeneralUtil.UpdatingTextField(
				GENO_DEFAULT.toString(), 10, null);
		hwe = new GeneralUtil.UpdatingTextField(
				10, null);
		mind = new GeneralUtil.UpdatingTextField(
				MIND_DEFAULT.toString(), 10, null);
		mendell = new GeneralUtil.UpdatingTextField(
				10, null);
		mendell2 = new GeneralUtil.UpdatingTextField(
				10, null);
		
		mafButton = new GeneralUtil.OptionalSelectButton("Minor allele frequency " +
				"(--maf)", maf, null);
		maxMafButton = new GeneralUtil.OptionalSelectButton("Maximum minor allele " +
				"frequency (--max-maf)", maxMaf, null);
		genoButton = new GeneralUtil.OptionalSelectButton("Maximum SNP missingness " +
				"rate (--geno)", geno, null);
		hweButton = new GeneralUtil.OptionalSelectButton("Hardy Weinberg equilibrium " +
				"(--hwe)", hwe, null);
		GeneralUtil.UpdatingTextField [] temp = {mendell, mendell2};
		mendellButton = new GeneralUtil.OptionalSelectButton("Mendel errors (--me)", 
				temp, null);
		mindButton = new GeneralUtil.OptionalSelectButton("Maximum individual " +
				"missingness rate (--mind)", mind, null);
		
	}
	
	/**
	 * Create a dialog to allow the user to specify threshold values
	 * @param givenParent form that this dialog is anchored to
	 */
	public ThresholdDialog(Forms givenParent){
		super(givenParent, "Threshold");
		parent = givenParent;
		
		initalize();
		
		getContentPane().setLayout(
				new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));
		
		JPanel mafPanel = new JPanel();
		mafPanel.setLayout(new FlowLayout(FlowLayout.LEADING));
		mafPanel.add(mafButton);
		mafPanel.add(maf);
		
		JPanel maxPanel = new JPanel();
		maxPanel.setLayout(new FlowLayout(FlowLayout.LEADING));
		maxPanel.add(maxMafButton);
		maxPanel.add(maxMaf);
		
		JPanel genoPanel = new JPanel();
		genoPanel.setLayout(new FlowLayout(FlowLayout.LEADING));
		genoPanel.add(genoButton);
		genoPanel.add(geno);
		
		JPanel mindPanel = new JPanel();
		mindPanel.setLayout(new FlowLayout(FlowLayout.LEADING));
		mindPanel.add(mindButton);
		mindPanel.add(mind);
		
		JPanel hwePanel = new JPanel();
		hwePanel.setLayout(new FlowLayout(FlowLayout.LEADING));
		hwePanel.add(hweButton);
		hwePanel.add(hwe);
		
		JPanel mendPanel = new JPanel();
		mendPanel.setLayout(new FlowLayout(FlowLayout.LEADING));
		mendPanel.add(mendellButton);
		mendPanel.add(mendell);
		mendPanel.add(mendell2);

		JPanel buttonPanel = new JPanel();
		okButton = new JButton("OK");
		okButton.addActionListener(process);
		buttonPanel.add(okButton);
		resetButton = new JButton("Restore Default");
		resetButton.addActionListener(reset);
		buttonPanel.add(resetButton);
		cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(cancel);
		buttonPanel.add(cancelButton);
		
		
		getContentPane().add(mafPanel);
		getContentPane().add(maxPanel);
		getContentPane().add(genoPanel);
		getContentPane().add(mindPanel);
		getContentPane().add(hwePanel);
		getContentPane().add(mendPanel);
		getContentPane().add(buttonPanel);
		
		pack();
		setVisible(true);
	}
}
